<?php

use Behat\Behat\Hook\Scope\AfterStepScope;
use Behat\Behat\Context\Context;
use Behat\Gherkin\Node\PyStringNode;
use Behat\Gherkin\Node\TableNode;

/**
 * Defines application features from the specific context.
 */
class FeatureContext extends \Behat\MinkExtension\Context\MinkContext implements Context
{
    /**
     * Initializes context.
     * Every scenario gets its own context instance.
     * You can also pass arbitrary arguments to the
     * context constructor through behat.yml.
     */
    public function __construct() {
    }

    /**
     * @BeforeScenario
     */
    public function beforeScenario(Behat\Behat\Hook\Scope\BeforeScenarioScope $scope) {
        //$this->getSession()->getDriver()->resizeWindow(1920, 2000);
    }

    /**
     * @When I wait :seconds seconds
     */
    public function iWaitSeconds($seconds) {
        $this->getSession()->wait(
            $seconds * 1000
        );
    }

    /**
     * @When I click on :name menu
     */
    public function iOpenMainMenu($name)
    {
        $this->clickLink($name);
        $this->getSession()->wait(
            2000, "$('.dropdown-menu:visible').css('opacity') == 1"
        );

    }

    /**
     * @AfterStep
     */
    public function takeScreenShotAfterFailedStep(afterStepScope $scope) {
        if (99 === $scope->getTestResult()->getResultCode()) {
//			..$driver = $this->getSession()->getDriver();

            //	mkdir('./debug_'.get_class($driver));

            //	if (!($driver instanceof ChromeDriver)) {
            //		return;
            //	}

            $screenshotData = $this->getSession()->getDriver()->getScreenshot();

            //if ($screenshotData) {
            $testrail_test_id = 0;
            foreach ($scope->getFeature()->getTags() as $tag) {
                $tokens = explode(" ", $tag);
                if ($tokens[0] == 'testrail_test_id') {
                    $testrail_test_id = $tokens[1];
                }
            }

            if (!is_dir('./screens')) {
                mkdir('./screens');
            }
            file_put_contents('./screens/test_'.uniqid().'.png', $screenshotData);
            //}
        }
    }

    /**
     * Click some text
     ** @Given I have a screen resolution of :width x :height
     */
    public function iHaveaScreenResolutionOf($width, $height) {
        $this->getSession()->getDriver()->resizeWindow((int)$width, (int)$height);
    }

    /**
     * Click some text
     * @When /^I click on the text "([^"]*)"$/
     */
    public function iClickOnTheText($text) {
        $session = $this->getSession();
        $element = $session->getPage()->find(
            'xpath',
            $session->getSelectorsHandler()->selectorToXpath('xpath', '*//*[text()="'.$text.'"]')
        );
        if (null === $element) {
            throw new \InvalidArgumentException(sprintf('Cannot find text: "%s"', $text));
        }

        $element->click();
    }

    /**
     * Click some text
     * @When /^I open the responsive main menu$/
     */
    public function iOpenTheResponsiveMainMenu() {
        $session = $this->getSession();
        $element = $session->getPage()->find('css', '#MainMenuBurgerButton');
        if (null !== $element && $element->isVisible()) {
            $element->click();

            // animation abwarten
            $this->getSession()->wait(
                500
            );
        }
    }

    /**
     * @Given I click the :arg1 element
     */
    public function iClickTheElement($selector) {
        $page = $this->getSession()->getPage();
        $element = $page->find('css', $selector);

        if (empty($element)) {
            throw new Exception("No html element found for the selector ('$selector')");
        }

        $element->click();
    }

    /**
     * @When I scroll :elementId into view
     */
    public function scrollIntoView($elementId) {
        $function = "";
        $function .= "(function(){";
        $function .= "var elem = $(':contains(\"".$elementId."\"), [value=\"".$elementId."\"]');";
        $function .= "$(elem)[0].scrollIntoView(false);";
        $function .= "})()";
        try {
            $this->getSession()->executeScript($function);
        }
        catch(Exception $e) {
            throw new \Exception("ScrollIntoView failed");
        }
    }

    /**
     * @Given I select :value in the :selector element
     */
    public function iSelectValueInTheElement($value, $selector) {
        $function = "";
        $function .= "(function(){";
        $function .= "var elem = $('".$selector." option[value=\"".$value."\"]').prop('selected','selected');";
        $function .= "$('".$selector."').multiselect(\"refresh\");";
        $function .= "})()";
        try {
            $this->getSession()->executeScript($function);
        }
        catch(Exception $e) {
            throw new \Exception("iSelectValueInTheElement failed".$e->getMessage());
        }
    }


    /**
     * @Given I select the ilSubTab :subTabSelector
     */
    public function iSelectTheIlsubtab($subTabSelector)
    {
        $page = $this->getSession()->getPage();
        $element = $page->find('css', $subTabSelector);

        if (empty($element)) {
            throw new Exception("No html element found for the selector ('$subTabSelector')");
        }

    }
}
